@@ -36,7 +36,7 @@ const SlideLayout = new Lang.Class({
 
     _init: function(params) {
         this._slideX = 1;
-        this._translationX = 0;
+        this._translationX = undefined;
         this._direction = SlideDirection.LEFT;
 
         this.parent(params);
@@ -161,7 +161,8 @@ const SlidingControl = new Lang.Class({
         let translationEnd = 0;
         let translation = this._getTranslation();
 
-        if (this._visible) {
+        let shouldShow = (this._getSlide() > 0);
+        if (shouldShow) {
             translationStart = translation;
         } else {
             translationEnd = translation;
@@ -177,8 +178,8 @@ const SlidingControl = new Lang.Class({
     },
 
     _onOverviewHiding: function() {
-        // We need to explicitily slideOut since showing pages
-        // doesn't implies to slide out, instead, hiding the overview does.
+        // We need to explicitly slideOut since showing pages
+        // doesn't imply sliding out, instead, hiding the overview does.
         this.slideOut();
     },
 
@@ -192,7 +193,7 @@ const SlidingControl = new Lang.Class({
 
     _onDragBegin: function() {
         this._inDrag = true;
-        this.layout.translationX = 0;
+        this._updateTranslation();
         this._updateSlide();
     },
 
@@ -217,7 +218,6 @@ const SlidingControl = new Lang.Class({
 
     slideIn: function() {
         this._visible = true;
-        this._updateTranslation();
         // we will update slideX and the translation from pageEmpty
     },
 
