@@ -367,10 +367,6 @@ const WorkspaceThumbnail = new Lang.Class({
         if (index == -1)
             return;
 
-        // Check if window still should be here
-        if (win && this._isMyWindow(win) && this._isOverviewWindow(win))
-            return;
-
         let clone = this._windows[index];
         this._windows.splice(index, 1);
 
